// This class creates a frame with a AnimationPanel.  It attaches a timer to
// the animation panel that generates an animation by asking the panel to
// repaint itself at regular intervals.  The frame also includes a slider that
// allows the user to ask the timer to speed up or slow down the rate at which
// it asks the panel to repaint.

import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;

public class AnimationFrame {
    private JFrame frame;
    private AnimationPanel panel;
    private Timer timer;

    // post: Constructs all of the objects needed for the animation: the frame,
    //       the panel, the slider and the timer.
    public AnimationFrame() {
        // set up frame
        frame = new JFrame();
        frame.setTitle("Fun with Animation");
        frame.setSize(300, 300);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // set up animation panel
        panel = new AnimationPanel();
        frame.add(panel, "Center");

        // set up timer and slider
        addTimer();
        addSlider();
    }

    // pre : all components have been initialized
    // post: begins the animation by making the frame visible and starting 
    //       the timer
    public void start() {
        frame.setVisible(true);
        timer.start();
    }

    // post: creates a timer that repaints the animation
    private void addTimer() {
        timer = new Timer(0, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                panel.repaint();
            }
        });
        timer.setCoalesce(true);
    }

    // post: creates a slider that allows the user to change the speed of the
    //       animation (frames per second, 1 to 30) and adds it to the southern
    //       part of the frame.
    private void addSlider() {
        final JSlider slider = new JSlider(1, 30);
        slider.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                timer.setDelay(1000 / slider.getValue());
            }
        });
        // start at 20 frames per second
        slider.setValue(20);

        // create a panel for the slider and its slow/fast labels
        JPanel p = new JPanel();
        p.add(new JLabel("slow"));
        p.add(slider);
        p.add(new JLabel("fast"));
        frame.add(p, "South");
    }
 }
